/* GtkDatabox - An extension to the gtk+ library
 * Copyright (C) 1998 - 2005  Dr. Roland Bock
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <gtkdatabox.h>

typedef struct
{
   gint v_lines;
   gint h_lines;
   GdkGC *data_gc;
   GdkGC *grid_gc;
}
GtkDataboxGridData;

void
gtk_databox_grid_draw (GtkDatabox * box, GtkDataboxData * data)
{
   gfloat x = 0;
   gfloat y = 0;
   gint i = 0;
   GtkDataboxGridData *grid_data;
   gfloat v_lines = 0; /* This is the default (no grid is drawn) */
   gfloat h_lines = 0; /* (use gtk_databox_data_set_grid_data)  */

   g_return_if_fail (data->draw_info != NULL);

   grid_data = (GtkDataboxGridData *) data->draw_info;
   
   v_lines = grid_data->v_lines;
   h_lines = grid_data->h_lines;

   /* We monitor whether the data->gc is new or has changed */
   if (grid_data->data_gc != data->gc) 
   {
      /* Then we create a modified gc for our purposes */
      grid_data->data_gc = data->gc;
      if (!grid_data->grid_gc) 
         grid_data->grid_gc = gdk_gc_new (box->pixmap);
      gdk_gc_copy(grid_data->grid_gc, data->gc);
      gdk_gc_set_line_attributes (grid_data->grid_gc, data->size, 
                                  GDK_LINE_ON_OFF_DASH, 
                                  GDK_CAP_BUTT, GDK_JOIN_MITER);
   }
   
   for (i = 0; i < v_lines; i++)
   {
      x = box->min.x + (gfloat)(i + 1) * (box->max.x - box->min.x) 
                                       / (v_lines + 1);
      x = (x - box->top_left.x) * box->factor.x;
      gdk_draw_line (box->pixmap, grid_data->grid_gc, x, 0, x, box->size.y);
   }
   for (i = 0; i < h_lines; i++)
   {
      y = box->min.y + (gfloat)(i + 1) * (box->max.y - box->min.y) 
                                       / (h_lines + 1);
      y = (y - box->top_left.y) * box->factor.y;
      gdk_draw_line (box->pixmap, grid_data->grid_gc, 0, y, box->size.x, y);
   }
}

gint
gtk_databox_grid_set_config (GtkDatabox * box, gint index, 
                                  guint hlines, guint vlines)
{
   GtkDataboxGridData * grid_data;
   GtkDataboxData *data = gtk_databox_data_get (box, index);

   g_return_val_if_fail (data, -1);
   if (data->draw_info == NULL) {
	   grid_data = g_new0 (GtkDataboxGridData, 1);
	   data->draw_info = grid_data;
   }else
	   grid_data = (GtkDataboxGridData *)data->draw_info;
   grid_data->v_lines = vlines;
   grid_data->h_lines = hlines;
   
   return 0;
}


gint
gtk_databox_grid_get_config (GtkDatabox * box, gint index,
                                  guint *hlines, guint *vlines)
{
   GtkDataboxGridData *grid_data;
   GtkDataboxData *data = gtk_databox_data_get (box, index);

   g_return_val_if_fail (data, -1);
   g_return_val_if_fail (data->draw_info, -1);

   grid_data = (GtkDataboxGridData *)data->draw_info;
   *vlines = grid_data->v_lines;
   *hlines = grid_data->h_lines;	
   return 0;

}

void
gtk_databox_grid_cleanup (GtkDatabox * box, GtkDataboxData * data)
{
   GtkDataboxGridData *grid_data;

   g_return_if_fail (data);
   g_return_if_fail (data->draw_info);
   
   grid_data = (GtkDataboxGridData *) data->draw_info;
   if (grid_data->grid_gc)
      g_object_unref (grid_data->grid_gc);
   
   g_free(data->draw_info);

   data->draw_info = NULL;
}
